
/*************************************************************************************************
MatrixKeyboard Keil uVision Plugin

Copyright (C) 2010 Ezequiel Gastn Miravalles

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*************************************************************************************************/

/*************************************************************************************************
Software: MatrixKeyboard Keil uVision Plugin
Author: Ezequiel Gastn Miravalles
Website: http://www.neoegm.com/software/keil-plugins/matrix-keyboard/
License: GNU GPL v3 (attached, read above)
Notes: AGSI library code adapted from Keil supplied samples
*************************************************************************************************/

#pragma once

#include "AGSI.h"

// Callback functions that are ONLY allowed to call during the initialisation process
typedef AGSIEXPORT BOOL      (*tAgsiDefineSFR) (const char* pszSfrName, AGSIADDR dwAddress, AGSITYPE eType, BYTE bBitPos);
typedef AGSIEXPORT AGSIVTR   (*tAgsiDefineVTR) (const char* pszVtrName, AGSITYPE eType, DWORD dwValue);
typedef AGSIEXPORT BOOL      (*tAgsiDeclareInterrupt)(AGSIINTERRUPT *pInterrupt);
typedef AGSIEXPORT BOOL      (*tAgsiSetWatchOnSFR) (AGSIADDR SFRAddress, AGSICALLBACK pfnReadWrite, AGSIACCESS eAccess);
typedef AGSIEXPORT BOOL      (*tAgsiSetWatchOnVTR) (AGSIVTR hVTR, AGSICALLBACK pfnReadWrite, AGSIACCESS eAccess);
typedef AGSIEXPORT BOOL      (*tAgsiSetWatchOnMemory) (AGSIADDR StartAddress, AGSIADDR EndAddress, AGSICALLBACK pfnReadWrite, AGSIACCESS eAccess);
typedef AGSIEXPORT AGSITIMER (*tAgsiCreateTimer) (AGSICALLBACK pfnTimer);
typedef AGSIEXPORT BOOL      (*tAgsiDefineMenuItem)(AGSIMENU *pDym);

// Callback functions that are NOT allowed to call during the initialisation process
typedef AGSIEXPORT BOOL      (*tAgsiWriteSFR) (AGSIADDR SFRAddress, DWORD dwValue, DWORD dwMask);
typedef AGSIEXPORT BOOL      (*tAgsiReadSFR)  (AGSIADDR SFRAddress, DWORD* pdwCurrentValue, DWORD* pdwPreviousValue, DWORD dwMask);
typedef AGSIEXPORT BOOL      (*tAgsiWriteVTR) (AGSIVTR hVTR, DWORD dwValue);
typedef AGSIEXPORT BOOL      (*tAgsiReadVTR) (AGSIVTR hVTR, DWORD* pdwCurrentValue);
typedef AGSIEXPORT BOOL      (*tAgsiSetSFRReadValue)(DWORD dwValue);
typedef AGSIEXPORT BOOL      (*tAgsiReadMemory) (AGSIADDR Address, DWORD dwCount, BYTE* pbValue);
typedef AGSIEXPORT BOOL      (*tAgsiWriteMemory)(AGSIADDR Address, DWORD dwCount, BYTE* pbValue);
typedef AGSIEXPORT AGSIADDR  (*tAgsiGetLastMemoryAddress)(void);
typedef AGSIEXPORT BOOL      (*tAgsiIsSimulatorAccess)(void);
typedef AGSIEXPORT BOOL      (*tAgsiSetTimer)(AGSITIMER hTimer, DWORD dwClock);
typedef AGSIEXPORT UINT64    (*tAgsiGetStates)(void);
typedef AGSIEXPORT AGSIADDR  (*tAgsiGetProgramCounter)(void);
typedef AGSIEXPORT DWORD     (*tAgsiIsInInterrupt)(void);
typedef AGSIEXPORT BOOL      (*tAgsiIsSleeping)(void);
typedef AGSIEXPORT void      (*tAgsiStopSimulator)(void);
typedef AGSIEXPORT void      (*tAgsiTriggerReset)(void);
typedef AGSIEXPORT void      (*tAgsiUpdateWindows)(void);
typedef AGSIEXPORT void      (*tAgsiHandleFocus) (HWND hwndDialog);
typedef AGSIEXPORT DWORD     (*tAgsiGetExternalClockRate)(void);
typedef AGSIEXPORT DWORD     (*tAgsiGetInternalClockRate)(void);
typedef AGSIEXPORT double    (*tAgsiGetClockFactor)(void);
typedef AGSIEXPORT void      (*tAgsiMessage)(const char* pszFormat, ...);
typedef AGSIEXPORT const char*(*tAgsiGetTargetKey)(const char* pszKey);
typedef AGSIEXPORT BOOL      (*tAgsiSetTargetKey)(const char* pszKey, const char *pszString);
typedef AGSIEXPORT DWORD     (*tAgsiGetSymbolByName) (AGSISYMDSC *vp);
typedef AGSIEXPORT DWORD     (*tAgsiGetSymbolByValue)(AGSISYMDSC *vp);

struct AGSIFUNCS {
	tAgsiDefineSFR             DefineSFR;
	tAgsiDefineVTR             DefineVTR;
	tAgsiSetWatchOnSFR         SetWatchOnSFR;
	tAgsiSetWatchOnVTR         SetWatchOnVTR;
	tAgsiSetWatchOnMemory      SetWatchOnMemory;
	tAgsiCreateTimer           CreateTimer;
	tAgsiDefineMenuItem        DefineMenuItem;
	tAgsiWriteSFR              WriteSFR;
	tAgsiReadSFR               ReadSFR;
	tAgsiWriteVTR              WriteVTR;
	tAgsiReadVTR               ReadVTR;
	tAgsiSetSFRReadValue       SetSFRReadValue;
	tAgsiReadMemory            ReadMemory;
	tAgsiWriteMemory           WriteMemory;
	tAgsiGetLastMemoryAddress  GetLastMemoryAddress;
	tAgsiIsSimulatorAccess     IsSimulatorAccess;
	tAgsiSetTimer              SetTimer;
	tAgsiGetStates             GetStates;
	tAgsiGetProgramCounter     GetProgramCounter;
	tAgsiDeclareInterrupt      DeclareInterrupt;
	tAgsiIsInInterrupt         IsInInterrupt;
	tAgsiIsSleeping            IsSleeping;
	tAgsiStopSimulator         StopSimulator;
	tAgsiTriggerReset          TriggerReset;
	tAgsiUpdateWindows         UpdateWindows;
	tAgsiHandleFocus           HandleFocus;
	tAgsiGetExternalClockRate  GetExternalClockRate;
	tAgsiGetInternalClockRate  GetInternalClockRate;
	tAgsiGetClockFactor        GetClockFactor;
	tAgsiMessage               Message;
	tAgsiGetTargetKey          GetTargetKey;
	tAgsiSetTargetKey          SetTargetKey;
	tAgsiGetSymbolByName       GetSymbolByName;
	tAgsiGetSymbolByValue      GetSymbolByValue;
};

extern struct AGSIFUNCS  Agsi;

union fv {      // float value union
	float f;
	DWORD DW;
};

// SFR's
#define IE       0xA8
#define IP       0xB8
#define IPH      0xB7
#define ADCCON1  0xEF
#define ADCCON2  0xD8
#define ADCCON3  0xF5
#define DMAL     0xD2
#define DMAH     0xD3
#define DMAP     0xD4
#define ADCDATAL 0xD9
#define ADCDATAH 0xDA

struct vtrlist {
	char     *pName;       // name of VTREG
	AGSITYPE Type;         // type of VTREG
	DWORD    Value;         // value of VTREG
	AGSIVTR  hVTR;         // handle of VTREG
};

// VTR's
extern struct vtrlist VTREG[];

extern AGSICONFIG  AgsiConfig;
extern AGSIFUNCS   Agsi;

DWORD GetFunctionPointers(void);
